@@ -1,10 +1,10 @@
 #!/bin/sh
 #
-# "$Id: makedocs.sh 189 2005-02-26 05:27:25Z mike $"
+# "$Id: makedocs.sh 297 2007-09-09 07:16:52Z mike $"
 #
 # Script to make documentation...
 #
-# Copyright 2003-2005 by Michael Sweet.
+# Copyright 2003-2007 by Michael Sweet.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -17,18 +17,14 @@
 # GNU General Public License for more details.
 #
 
-htmldoc --verbose --batch mxml.book --no-embedfonts -f mxml.pdf
+htmldoc --verbose --path "hires;." --batch mxml.book -f mxml.pdf
 
-htmldoc --verbose --batch mxml.book -f mxml.html
-
-htmldoc --verbose --batch mxml.book --no-embedfonts -f mxml.ps
-rm -f mxml.ps.gz
-gzip -v9 mxml.ps
+htmldoc --verbose --batch mxml.book --no-title -f mxml.html
 
 rm -rf mxml.d
 mkdir mxml.d
-htmldoc --verbose --batch mxml.book -t htmlsep -d mxml.d
+htmldoc --verbose --batch mxml.book --no-title -t html -d mxml.d
 
 #
-# End of "$Id: makedocs.sh 189 2005-02-26 05:27:25Z mike $".
+# End of "$Id: makedocs.sh 297 2007-09-09 07:16:52Z mike $".
 #
