@@ -1,9 +1,9 @@
 /*
- * "$Id: mxml-node.c 184 2005-01-29 07:21:44Z mike $"
+ * "$Id: mxml-node.c 270 2007-04-23 21:48:03Z mike $"
  *
  * Node support code for Mini-XML, a small XML-like file parsing library.
  *
- * Copyright 2003-2005 by Michael Sweet.
+ * Copyright 2003-2007 by Michael Sweet.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,13 +19,18 @@
  *
  *   mxmlAdd()        - Add a node to a tree.
  *   mxmlDelete()     - Delete a node and all of its children.
+ *   mxmlNewCDATA()   - Create a new CDATA node.
+ *   mxmlNewCustom()  - Create a new custom data node.
  *   mxmlNewElement() - Create a new element node.
  *   mxmlNewInteger() - Create a new integer node.
  *   mxmlNewOpaque()  - Create a new opaque string.
  *   mxmlNewReal()    - Create a new real number node.
  *   mxmlNewText()    - Create a new text fragment node.
  *   mxmlNewTextf()   - Create a new formatted text fragment node.
+ *   mxmlNewXML()     - Create a new XML document tree.
+ *   mxmlRelease()    - Release a node.
  *   mxmlRemove()     - Remove a node from its parent.
+ *   mxmlRetain()     - Retain a node.
  *   mxml_new()       - Create a new node.
  */
 
@@ -261,6 +266,8 @@ mxmlDelete(mxml_node_t *node)		/* I - Node to delete */
 	    node->value.custom.destroy)
 	  (*(node->value.custom.destroy))(node->value.custom.data);
 	break;
+    default :
+        break;
   }
 
  /*
@@ -271,6 +278,47 @@ mxmlDelete(mxml_node_t *node)		/* I - Node to delete */
 }
 
 
+/*
+ * 'mxmlNewCDATA()' - Create a new CDATA node.
+ *
+ * The new CDATA node is added to the end of the specified parent's child
+ * list. The constant MXML_NO_PARENT can be used to specify that the new
+ * CDATA node has no parent. The data string must be nul-terminated and
+ * is copied into the new node. CDATA nodes use the MXML_ELEMENT type.
+ *
+ * @since Mini-XML 2.3@
+ */
+
+mxml_node_t *				/* O - New node */
+mxmlNewCDATA(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
+	     const char  *data)		/* I - Data string */
+{
+  mxml_node_t	*node;			/* New node */
+
+
+#ifdef DEBUG
+  fprintf(stderr, "mxmlNewCDATA(parent=%p, data=\"%s\")\n",
+          parent, data ? data : "(null)");
+#endif /* DEBUG */
+
+ /*
+  * Range check input...
+  */
+
+  if (!data)
+    return (NULL);
+
+ /*
+  * Create the node and set the name value...
+  */
+
+  if ((node = mxml_new(parent, MXML_ELEMENT)) != NULL)
+    node->value.element.name = _mxml_strdupf("![CDATA[%s]]", data);
+
+  return (node);
+}
+
+
 /*
  * 'mxmlNewCustom()' - Create a new custom data node.
  *
@@ -278,13 +326,15 @@ mxmlDelete(mxml_node_t *node)		/* I - Node to delete */
  * list. The constant MXML_NO_PARENT can be used to specify that the new
  * element node has no parent. NULL can be passed when the data in the
  * node is not dynamically allocated or is separately managed.
+ *
+ * @since Mini-XML 2.1@
  */
 
 mxml_node_t *				/* O - New node */
-mxmlNewCustom(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
-              void        *data,	/* I - Pointer to data */
-	      void        (*destroy)(void *))
-					/* I - Function to destroy data */
+mxmlNewCustom(
+    mxml_node_t              *parent,	/* I - Parent node or MXML_NO_PARENT */
+    void                     *data,	/* I - Pointer to data */
+    mxml_custom_destroy_cb_t destroy)	/* I - Function to destroy data */
 {
   mxml_node_t	*node;			/* New node */
 
@@ -365,13 +415,6 @@ mxmlNewInteger(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
   fprintf(stderr, "mxmlNewInteger(parent=%p, integer=%d)\n", parent, integer);
 #endif /* DEBUG */
 
- /*
-  * Range check input...
-  */
-
-  if (!parent)
-    return (NULL);
-
  /*
   * Create the node and set the element name...
   */
@@ -408,7 +451,7 @@ mxmlNewOpaque(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
   * Range check input...
   */
 
-  if (!parent || !opaque)
+  if (!opaque)
     return (NULL);
 
  /*
@@ -441,13 +484,6 @@ mxmlNewReal(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
   fprintf(stderr, "mxmlNewReal(parent=%p, real=%g)\n", parent, real);
 #endif /* DEBUG */
 
- /*
-  * Range check input...
-  */
-
-  if (!parent)
-    return (NULL);
-
  /*
   * Create the node and set the element name...
   */
@@ -486,7 +522,7 @@ mxmlNewText(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
   * Range check input...
   */
 
-  if (!parent || !string)
+  if (!string)
     return (NULL);
 
  /*
@@ -532,7 +568,7 @@ mxmlNewTextf(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
   * Range check input...
   */
 
-  if (!parent || !format)
+  if (!format)
     return (NULL);
 
  /*
@@ -544,7 +580,7 @@ mxmlNewTextf(mxml_node_t *parent,	/* I - Parent node or MXML_NO_PARENT */
     va_start(ap, format);
 
     node->value.text.whitespace = whitespace;
-    node->value.text.string     = mxml_strdupf(format, ap);
+    node->value.text.string     = _mxml_vstrdupf(format, ap);
 
     va_end(ap);
   }
@@ -620,6 +656,71 @@ mxmlRemove(mxml_node_t *node)		/* I - Node to remove */
 }
 
 
+/*
+ * 'mxmlNewXML()' - Create a new XML document tree.
+ *
+ * The "version" argument specifies the version number to put in the
+ * ?xml element node. If NULL, version 1.0 is assumed.
+ *
+ * @since Mini-XML 2.3@
+ */
+
+mxml_node_t *				/* O - New ?xml node */
+mxmlNewXML(const char *version)		/* I - Version number to use */
+{
+  char	element[1024];			/* Element text */
+
+
+  snprintf(element, sizeof(element), "?xml version=\"%s\"?",
+           version ? version : "1.0");
+
+  return (mxmlNewElement(NULL, element));
+}
+
+
+/*
+ * 'mxmlRelease()' - Release a node.
+ *
+ * When the reference count reaches zero, the node (and any children)
+ * is deleted via mxmlDelete().
+ *
+ * @since Mini-XML 2.3@
+ */
+
+int					/* O - New reference count */
+mxmlRelease(mxml_node_t *node)		/* I - Node */
+{
+  if (node)
+  {
+    if ((-- node->ref_count) <= 0)
+    {
+      mxmlDelete(node);
+      return (0);
+    }
+    else
+      return (node->ref_count);
+  }
+  else
+    return (-1);
+}
+
+
+/*
+ * 'mxmlRetain()' - Retain a node.
+ *
+ * @since Mini-XML 2.3@
+ */
+
+int					/* O - New reference count */
+mxmlRetain(mxml_node_t *node)		/* I - Node */
+{
+  if (node)
+    return (++ node->ref_count);
+  else
+    return (-1);
+}
+
+
 /*
  * 'mxml_new()' - Create a new node.
  */
@@ -656,7 +757,8 @@ mxml_new(mxml_node_t *parent,		/* I - Parent node */
   * Set the node type...
   */
 
-  node->type = type;
+  node->type      = type;
+  node->ref_count = 1;
 
  /*
   * Add to the parent if present...
@@ -674,5 +776,5 @@ mxml_new(mxml_node_t *parent,		/* I - Parent node */
 
 
 /*
- * End of "$Id: mxml-node.c 184 2005-01-29 07:21:44Z mike $".
+ * End of "$Id: mxml-node.c 270 2007-04-23 21:48:03Z mike $".
  */
